@@ -16,7 +16,7 @@
 #' }
 src_duckdb <- function(
     drv = duckdb::duckdb(),
-    dbdir = ':memory:',
+    dbdir = attr(drv, "dbdir"),
     ...) {
 
   con <- duckdb::dbConnect(
@@ -25,6 +25,13 @@ src_duckdb <- function(
     ...
     )
 
+  # ensure disconnect
+  reg.finalizer(
+    e = globalenv(),
+    f = closeNodbiConnections,
+    onexit = TRUE
+  )
+
   # potential security concern with
   # storing the full connection string
   structure(list(con = con,
